/*
 * File:       ControlCenter.h
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    Mon Apr 21 1997
 * Revision:   $Revision: 1.6 $
 * Updated:    $Date: 1998/07/07 12:21:22 $
 * Copyright 1997 NPAC, Syracuse University
 *
 */

#ifndef _CONTROLCENTER_H
#define _CONTROLCENTER_H

#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>

#include <ace/XtReactor.h>
#include <ace/Map_Manager.h>

#include "TangoLinkHandler.h"
#include "ViewerLinkHandler.h"
#include "IDrep.h"

#include <GeoPoint.h>

class ControlCenter
{
   public:
      enum Mode { MASTER, SLAVE };
      typedef unsigned short AreaParams[4];
      typedef TangoLinkHandler::IPC_SAP TANGO_SAP;
      typedef ViewerLinkHandler::IPC_SAP VIEWER_SAP;
      typedef MsgHeader<VIEWER_SAP>::VID ViewerLinkID;
      typedef ACE_Map_Manager<ViewerLinkID, ViewerLinkHandler*, ACE_Null_Mutex> ViewerLinks;
      typedef IDrep<ViewerLinkID> VIDRep;
      
      ControlCenter( int argc, char* argv[] );
      void setMode( Mode );
      void connClosed( void );
      void viewerClosed( ViewerLinkHandler* );
      void viewerClosed( const ViewerLinkID& vid );
      void synchronize( const ViewerLinkID& vid );
      void setVIDRep( ViewerLinkID& vid );
      void sendSynchro( void );
      void setSelArea( const AreaParams& ap );
      void getSelArea( AreaParams& ap );
      int sendToViewers( MsgHeader<TANGO_SAP>&, ACE_Message_Block& );
      int sendToTango( MsgHeader<VIEWER_SAP>&, ACE_Message_Block&, ViewerLinkHandler* );
      int spawnNewViewer( ViewerLinkID = 0 );
      Widget getWidget() { return top_level; };
      
      
   protected:
      int paramsInit( void );
      int guiInit( int argc, char* argv[] );
      int reactorInit( int argc, char* argv[] );
      void run( void );
      int viewerSynch( const ViewerLinkID& vid );
      
   private:
      static void launchCB( Widget, ControlCenter*, XtPointer );
      static void quitCB( Widget, ControlCenter*, XtPointer );
      static void exposeCB( Widget, ControlCenter*, XtPointer );
      static void mapCB( Widget, ControlCenter*, XmDrawingAreaCallbackStruct* );

      ViewerLinks viewerLinkRep;
      VIDRep viewerIDRep;
      Mode mode;
      Widget top_level, map_widget, launch_button;
      ACE_XtReactor* reactor;
      TangoLinkHandler* mainLink;

      const char* pathToGIS;
      const char* pathToMap;
      bool displayMap;
      
      Display* disp;
      GC map_gc;
      Colormap cmap;
      XColor selColor, gridColor, unusedColor;
      int scr;
      unsigned short mapWidth, mapHeight; //map size measured in chunks
      unsigned short nwLat, nwLon; //northwest corner of the map in minutes
      unsigned short chunkSzMin; //chunk size in minutes
      unsigned short chunkHorPix, chunkVerPix; //chunk sizes in pixels
      unsigned short area_ulx, area_uly, area_width, area_height; //selected area in pixels
      unsigned short bitmapWidth, bitmapHeight; //image sizes
};

inline int ControlCenter::
reactorInit( int, char* [] )
{
   ACE_TRACE( "(ControlCenter::reactorInit)" );
   reactor = new ACE_XtReactor( XtWidgetToApplicationContext( top_level ) );
   if( reactor->register_handler( mainLink, ACE_Event_Handler::READ_MASK ) == -1 )
     {
       cout << "registering failed" << endl;
       return -1;
     }

   return 0;
}

inline void ControlCenter::
run( void )
{
   ACE_TRACE( "(ControlCenter::run)" );
   XtRealizeWidget( top_level );
   XtAppMainLoop( XtWidgetToApplicationContext( top_level ) );
}

inline ControlCenter::
ControlCenter( int argc, char* argv[] )
   : viewerLinkRep( 3 ), mapWidth( 0 ), mapHeight( 0 ),
     nwLat( 0 ), nwLon( 0 ), chunkSzMin( 0 ), chunkHorPix( 0 ), chunkVerPix( 0 ),
     area_ulx( 0 ), area_uly( 0 ), area_width( 0 ), area_height( 0 ),
     bitmapWidth( 0 ), bitmapHeight( 0 ), displayMap( true )
{
   ACE_TRACE( "(ControlCenter::ControlCenter)" );
   signal( SIGCHLD, SIG_IGN );
   paramsInit();
   guiInit( argc, argv );
   int handle = registerApp( argc, argv );
   mainLink = new TangoLinkHandler( *this, handle );
   reactorInit( argc, argv );
   run();
}

inline void ControlCenter::
setMode( Mode m )
{
   ACE_TRACE( "(ControlCenter::setMode)" );
   ACE_DEBUG(( LM_DEBUG, "%s mode\n", m == MASTER ? "master" : "slave" ));
   mode = m;
   XtVaSetValues( launch_button,
		  XmNsensitive, ( m == MASTER ) ? TRUE : FALSE,
		  NULL );
}

inline void ControlCenter::
connClosed( void )
{
   ACE_DEBUG(( LM_SHUTDOWN, "(ControlCenter::connClosed) connection reset by peer\n" ));
   exit( 0 );
}

inline void ControlCenter::
synchronize( const ViewerLinkID& vid )
{
   ACE_TRACE( "(ControlCenter::synchronize)" );
   for( ViewerLinkID i=1; i<viewerIDRep.space(); i<<=1 )
      if( vid & i )
	 spawnNewViewer( i );
}

inline void ControlCenter::
setVIDRep( ViewerLinkID& vid )
{
   ACE_TRACE( "(ControlCenter::setVIDRep)" );
   vid = viewerIDRep.bitvec();
}

inline void ControlCenter::
getSelArea( AreaParams& ap )
{
   ACE_TRACE( "(ControlCenter::getSelArea)" );
   ap[0] = area_ulx;
   ap[1] = area_uly;
   ap[2] = area_width;
   ap[3] = area_height;
}

#endif
